% solve ODE problem (initial value problem)
% y'=A*y+b, y(0)=y0

clc;
close all;



% parameters
X=1;Y=1; % amplitudes
% change omega, observe the result
omega=1; % angular velocity


y0=[0;1]; % [sin(om*t);cos(om*t)] initial voltage on capacitance,initial current in circuit

% y'=A*y+b
A=[0, omega; 
   -omega, 0]; 
b=@(t) [0; 0];
% b=[0;0];

% simulation parameters
dt=0.1; % integration step size
tmax=50.0;
tspan=[0,tmax]; % time of simulation


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% numerical solutions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% y'=A*y+b
myodefun = @(t,y) A*y+b(t);
% myodefun = @(t,y) A*y+b;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MATLAB ode solvers
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% default options
% RelTol, AbsTol default: '1e-3'
%% try uncommenting the other line, observe the differences
options = odeset('Stats','on')
%options = odeset('RelTol',1e-10,'AbsTol',1e-10,'Stats','on');

tic
%[T_ode23,Y_ode23] = ode23(myodefun,tspan,y0);
[T_ode23,Y_ode23]=ode23(myodefun,tspan,y0,options); %[sin, cos]
disp('ode23') 
toc

tic
%[T_ode45,Y_ode45] = ode45(myodefun,tspan,y0);
[T_ode45,Y_ode45]=ode45(myodefun,tspan,y0,options); %[sin, cos]
disp('ode45') 
toc

NAME = sprintf('ode23_numSol_omega_%s',num2str(omega))
f = figure;
plot(T_ode23,Y_ode23(:,1),T_ode23,Y_ode23(:,2))
title('MATLAB - numerical solution ode23')
xlabel('time')
legend('sin(\omega t)','cos(\omega*t)')
saveas(f,NAME,'png');

NAME = sprintf('ode45_numSol_omega_%s',num2str(omega))
f = figure;
plot(T_ode45,Y_ode45(:,1),T_ode45,Y_ode45(:,2))
title('MATLAB - numerical solution ode45')
xlabel('time')
legend('sin(\omega t)','cos(\omega*t)')
saveas(f,NAME,'png');

%%% Phase plane
NAME = sprintf('ode23_circle_omega_%s',num2str(omega))
f = figure;
plot(Y_ode23(:,1),Y_ode23(:,2),'o')
title('ode23: Circuit test - phase plane')
xlabel('sin(\omega*t)')
ylabel('cos(\omega*t)')
saveas(f,NAME,'png');

NAME = sprintf('ode45_circle_omega_%s',num2str(omega))
f = figure;
plot(Y_ode45(:,1),Y_ode45(:,2),'o')
title('ode45: Circuit test - phase plane')
xlabel('sin(\omega*t)')
ylabel('cos(\omega*t)')
saveas(f,NAME,'png');



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Taylor series
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
eps=10^-7; % set error
tic
[T_Taylor, Y_Taylor, ORD, DY]=explicitTaylorLinear(dt,tspan,y0,A,b,eps);
disp('Taylor') 
toc

NAME = sprintf('mtsm_numSol_omega_%s',num2str(omega));
f = figure;
plot(T_Taylor,Y_Taylor(:,1),T_Taylor,Y_Taylor(:,2))
title('Taylor series numerical solution')
xlabel('time')
legend('sin(\omega t)','cos(\omega*t)')
saveas(f,NAME,'png');

%%% Phase plane
NAME = sprintf('mtsm_circle_omega_%s',num2str(omega));
f = figure;
plot(Y_Taylor(:,1),Y_Taylor(:,2),'o')
title('Taylor: Circuit test - phase plane')
xlabel('sin(\omega*t)')
ylabel('cos(\omega*t)')
saveas(f,NAME,'png');


NAME = sprintf('mtsm_order_omega_%s',num2str(omega));
f = figure;
plot(T_Taylor(2:end),ORD(2:end),'X')
title('Taylor series based method: Order')
xlabel('time')
ylabel('ORD')
axis([0 tmax 0 max(ORD)+1])
saveas(f,NAME,'png');



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Analytic solutions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x_anal=@(t) X*sin(omega*t);
y_anal=@(t) Y*cos(omega*t);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Absolute errors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
error_ode23_sin=abs(Y_ode23(:,1)-x_anal(T_ode23)); % error uc
error_ode23_cos=abs(Y_ode23(:,2)-y_anal(T_ode23)); % error i
error_ode45_sin=abs(Y_ode45(:,1)-x_anal(T_ode45)); % error uc
error_ode45_cos=abs(Y_ode45(:,2)-y_anal(T_ode45)); % error i
error_Taylor_sin=abs(Y_Taylor(:,1)-x_anal(T_Taylor'));
error_Taylor_cos=abs(Y_Taylor(:,2)-y_anal(T_Taylor'));

% figure
% plot(T_ode23,error_ode23_sin,T_Taylor,error_Taylor_sin)
% title('Absolute error of numerical solutions: sin(\omega*t)')
% legend('ode23 solver', 'Taylor series')
% 
% figure
% plot(T_ode23,error_ode23_cos,T_Taylor,error_Taylor_cos)
% title('Absolute error of numerical solutions: cos(\omega*t)')
% legend('ode23 solver', 'Taylor series')

NAME = sprintf('ode23_absErr_omega_%s',num2str(omega));
f = figure;
plot(T_ode23,error_ode23_sin,T_ode23,error_ode23_cos)
title('ode23 - absolute error of numerical solutions')
legend('sin(\omega*t)', 'cos(\omega*t)')
saveas(f,NAME,'png');

NAME = sprintf('ode45_absErr_omega_%s',num2str(omega));
f = figure;
plot(T_ode45,error_ode45_sin,T_ode45,error_ode45_cos)
title('ode45 - absolute error of numerical solutions')
legend('sin(\omega*t)', 'cos(\omega*t)')
saveas(f,NAME,'png');

NAME = sprintf('mtsm_absErr_omega_%s',num2str(omega));
f = figure;
plot(T_Taylor,error_Taylor_sin,T_Taylor,error_Taylor_cos)
title('Taylor: absolute error of numerical solutions')
legend('sin(\omega*t)', 'cos(\omega*t)')
saveas(f,NAME,'png');








fprintf('\nAbsolute errors:\n')
fprintf('MATLAB ode23 solver: %g \n',norm(error_ode23_sin+error_ode23_cos,inf));
fprintf('MATLAB ode45 solver: %g \n',norm(error_ode45_sin+error_ode45_cos,inf));
fprintf('Taylor series method: %g \n',norm(error_Taylor_sin+error_Taylor_cos,inf));




