% solve ODE problem (initial value problem)
% y'=A*y+b, y(0)=y0

% clc;
close all;

% RLC circuit
% uc'=(1/C)*i, uc(0)=0
% i'=(1/L)*(u-R*i-uc), i(0)=0

% parameters
R=200; % resistance
L=10^-2; % inductance
C=10^-6; % capacitance
U=1; % input voltage
omega=1000;
% omega=1/sqrt(L*C); % resonance - Thompson equation
u=@(t) U*sin(omega*t);

y0_Taylor=[0;0;0;U]; % [uc0;i0;U*sin(om*t);U*cos(om*t)] initial voltage on capacitance,initial current in circuit

% y'=A*y+b
A_Taylor=[0, 1/C, 0, 0; 
   -1/L, -R/L, (U/L), 0;
   0, 0, 0, omega
   0, 0, -omega, 0;]; 
b_Taylor=@(t) [0; 0; 0; 0];

y0=[0;0]; % [uc0;i0;sin(om*t);cos(om*t)] initial voltage on capacitance,initial current in circuit
A=[0, 1/C; 
   -1/L, -R/L;]; 
b=@(t) [0; u(t)/L]; 

% simulation parameters
dt=0.001; % integration step size
tmax=0.01;
tspan=[0,tmax]; % time of simulation


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% numerical solutions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% y'=A*y+b
myodefun = @(t,y) A*y+b(t);

% Ohm's law
ur=@(i) R*i; 
% II. Kirchhoff's law
ul=@(uc,i,t) u(t)-uc-R*i; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MATLAB ode solvers
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  tol=1e-7;
%  ABSTOL(1:length(y0))=tol;
%  options = odeset('RelTol',tol,'AbsTol',ABSTOL);
% options = odeset('RelTol',1e-13,'AbsTol',1e-13,'Stats','on','OutputFcn',@odeplot) 
options = [];

tic
% [T_ode23,Y_ode23]=ode23(myodefun,tspan,y0,options); %[uc, i]
[T_ode23,Y_ode23] = ode23(myodefun,tspan,y0);
disp('ode23') 
toc


UR=ur(Y_ode23(:,2));
UL=ul(Y_ode23(:,1),Y_ode23(:,2),T_ode23);

figure
subplot(2,1,1)
plot(T_ode23,Y_ode23(:,1),T_ode23,UR,T_ode23,UL,T_ode23,Y_ode23(:,2))
title('MATLAB - numerical solution ode23')
xlabel('time')
legend('u_C','u_R','u_L','i')

%%% Control computation: UL+UR+UC=U
% norm(UR+UL+Y_ode23(:,1)-U)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Taylor series
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
eps=10^-15; % set error
tic
[T_Taylor, Y_Taylor, ORD, DY]=explicitTaylorLinear(dt,tspan,y0_Taylor,A_Taylor,b_Taylor,eps);
disp('Taylor') 
toc

UR=ur(Y_Taylor(:,2));
UL=ul(Y_Taylor(:,1),Y_Taylor(:,2),T_Taylor');

% figure
subplot(2,1,2)
plot(T_Taylor,Y_Taylor(:,1),T_Taylor,UR,T_Taylor,UL,T_Taylor,Y_Taylor(:,2))
title('Taylor series numerical solution')
xlabel('time')
legend('u_C','u_R','u_L','i')

%%% Control computation: UL+UR+UC=U
% norm(UR+UL+Y_Taylor(:,1)-U)

figure
plot(T_Taylor(2:end),ORD(2:end),'X')
title('Taylor series based method: Order')
xlabel('time')
ylabel('ORD')
axis([0 tmax 0 max(ORD)+1])



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Analytic solutions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% integration constants
C1=2*10^7/10201-198*10^4/2040.2;
C2=2000/10201;
A=1980/2040.2;
B=-2000/10201;

% uc_anal=inline('exp(-10^4*t).*(C1*t+C2)+A*sin(omega*t)+B*cos(omega*t)','t','C1','C2','A','B','omega');
uc_anal=@(t) exp(-10^4*t).*(C1*t+C2*ones(length(t),1))+A*sin(omega*t)+B*cos(omega*t);
i=@(t) C*(-10^4*exp(-10^4*t).*(C1*t+C2*ones(length(t),1))+exp(-10^4*t)*C1+omega*A*cos(omega*t)-omega*B*sin(omega*t));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Absolute errors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
error_ode23=abs(Y_ode23(:,1)-uc_anal(T_ode23)); % error uc
error_ode23_i=abs(Y_ode23(:,2)-i(T_ode23)); % error i
error_Taylor=abs(Y_Taylor(:,1)-uc_anal(T_Taylor'));
error_Taylor_i=abs(Y_Taylor(:,2)-i(T_Taylor'));

figure
subplot(2,1,1)
plot(T_ode23,error_ode23,T_Taylor,error_Taylor)
title('Absolute error of numerical solutions: u_c')
legend('ode23 solver', 'Taylor series')

% figure
subplot(2,1,2)
plot(T_ode23,error_ode23_i,T_Taylor,error_Taylor_i)
title('Absolute error of numerical solutions: i')
legend('ode23 solver', 'Taylor series')

fprintf('\nAbsolute errors:\n')
fprintf('MATLAB ode23 solver: %g \n',norm(error_ode23+error_ode23_i,inf));
fprintf('Taylor series method: %g \n',norm(error_Taylor+error_Taylor_i,inf));

